@@ -1,20 +1,27 @@
+# Copyright (c) Jupyter Development Team.
+# Distributed under the terms of the Modified BSD License.
+
 from .widget import Widget, CallbackDispatcher, register, widget_serialization, handle_version_comm_opened
 from .domwidget import DOMWidget
+from .valuewidget import ValueWidget
 
-from .trait_types import Color, EventfulDict, EventfulList
+from .trait_types import Color, Datetime
 
+from .widget_core import CoreWidget
 from .widget_bool import Checkbox, ToggleButton, Valid
-from .widget_button import Button
-from .widget_box import Box, FlexBox, Proxy, PlaceProxy, HBox, VBox
+from .widget_button import Button, ButtonStyle
+from .widget_box import Box, HBox, VBox
 from .widget_float import FloatText, BoundedFloatText, FloatSlider, FloatProgress, FloatRangeSlider
 from .widget_image import Image
-from .widget_int import IntText, BoundedIntText, IntSlider, IntProgress, IntRangeSlider, Play
+from .widget_int import IntText, BoundedIntText, IntSlider, IntProgress, IntRangeSlider, Play, SliderStyle
 from .widget_color import ColorPicker
+from .widget_date import DatePicker
 from .widget_output import Output
 from .widget_selection import RadioButtons, ToggleButtons, Dropdown, Select, SelectionSlider, SelectMultiple
 from .widget_selectioncontainer import Tab, Accordion
-from .widget_string import HTML, Label, Latex, Text, Textarea
+from .widget_string import HTML, HTMLMath, Label, Text, Textarea
 from .widget_controller import Controller
-from .interaction import interact, interactive, fixed, interact_manual
+from .interaction import interact, interactive, fixed, interact_manual, interactive_output
 from .widget_link import jslink, jsdlink
 from .widget_layout import Layout
+from .widget_style import Style
