@@ -1,11 +1,6 @@
 var CodeMirror = require("codemirror");
 require("codemirror/lib/codemirror.css");
 require("codemirror/mode/python/python");
-
-require('bootstrap/dist/css/bootstrap.css');
-require('jquery-ui/themes/smoothness/jquery-ui.min.css');
-require("jupyter-js-widgets/css/widgets.min.css");
-
 var WidgetManager = require("./manager").WidgetManager;
 
 document.addEventListener("DOMContentLoaded", function(event) {
@@ -24,5 +19,9 @@ document.addEventListener("DOMContentLoaded", function(event) {
     var state = require("./widget_state.json");
     var widgetarea = document.getElementsByClassName("widgetarea")[0];
     var manager = new WidgetManager(widgetarea);
-    manager.set_state(state);
+    manager.set_state(state).then(function (models) {
+        manager.display_model(undefined, models.find(function(element) {
+            return element.id == "4c5a49e52ae54ad5bdbe973353a58931";
+        }));
+    });
 });
