@@ -1,5 +1,5 @@
 module.exports = {
-    entry: './es5/index.js',
+    entry: './lib/index.js',
     output: {
         filename: 'index.built.js',
         path: './built/',
@@ -7,7 +7,7 @@ module.exports = {
     },
     module: {
         loaders: [
-            { test: /\.css$/, loader: "style-loader!css-loader" },
+            { test: /\.css$/, loader: "style-loader!css-loader!postcss-loader" },
             { test: /\.json$/, loader: "json-loader" },
             // jquery-ui loads some images
             { test: /\.(jpg|png|gif)$/, loader: "file" },
@@ -18,5 +18,11 @@ module.exports = {
             { test: /\.eot(\?v=\d+\.\d+\.\d+)?$/, loader: "file" },
             { test: /\.svg(\?v=\d+\.\d+\.\d+)?$/, loader: "url?limit=10000&mimetype=image/svg+xml" }
         ]
+    },
+    postcss: function () {
+        return [
+            require('postcss-import'),
+            require('postcss-cssnext')
+        ];
     }
 };
