@@ -1,16 +1,19 @@
+// Copyright (c) Jupyter Development Team.
+// Distributed under the terms of the Modified BSD License.
+
 var version = require('./package.json').version;
 
 module.exports = {
-    entry: './src/embed-webpack.js',
+    entry: './lib-embed/embed-webpack.js',
     output: {
         filename: 'embed.js',
         path: './dist',
-        publicPath: 'https://npmcdn.com/jupyter-js-widgets@' + version + '/dist/'
+        publicPath: 'https://unpkg.com/jupyter-js-widgets@' + version + '/dist/'
     },
     devtool: 'source-map',
     module: {
         loaders: [
-            { test: /\.css$/, loader: "style-loader!css-loader" },
+            { test: /\.css$/, loader: "style-loader!css-loader!postcss-loader" },
             { test: /\.json$/, loader: "json-loader" },
             // jquery-ui loads some images
             { test: /\.(jpg|png|gif)$/, loader: "file" },
@@ -21,5 +24,11 @@ module.exports = {
             { test: /\.eot(\?v=\d+\.\d+\.\d+)?$/, loader: "file" },
             { test: /\.svg(\?v=\d+\.\d+\.\d+)?$/, loader: "url?limit=10000&mimetype=image/svg+xml" }
         ]
+    },
+    postcss: function () {
+        return [
+            require('postcss-import'),
+            require('postcss-cssnext')
+        ]
     }
 };
