@@ -5,15 +5,10 @@
 var _ = require("underscore");
 var Backbone = require("backbone");
 var widgets = require("jupyter-js-widgets");
-var html2canvas = require("html2canvas");
-var progressModal = require("./progress-modal");
 var saveState = require("./save_state");
 var embedWidgets = require("./embed_widgets");
 var version = require("../package.json").version;
-
-
-// Work around for a logging bug, reported in https://github.com/niklasvh/html2canvas/issues/543
-window.html2canvas = html2canvas;
+var output = require("./widget_output");
 
 //--------------------------------------------------------------------
 // WidgetManager class
@@ -30,8 +25,7 @@ var WidgetManager = function (comm_manager, notebook) {
     // Register with the comm manager.
     this.comm_manager.register_target(this.comm_target_name, _.bind(this.handle_comm_open,this));
 
-    // Attempt to reconstruct any live comms by requesting them from the
-    // back-end.
+    // Attempt to reconstruct any live comms by requesting them from the back-end.
     var that = this;
     var backed_widgets_loaded = this._get_comm_info().then(function(comm_ids) {
 
@@ -46,9 +40,8 @@ var WidgetManager = function (comm_manager, notebook) {
             return Promise.all(comms.map(function(comm) {
                 var update_promise = new Promise(function(resolve, reject) {
                     comm.on_msg(function (msg) {
-
                         // A suspected response was received, check to see if
-                        // it's a state update.  If so, resolve.
+                        // it's a state update. If so, resolve.
                         if (msg.content.data.method === 'update') {
                             resolve({
                                 comm: comm,
@@ -57,7 +50,6 @@ var WidgetManager = function (comm_manager, notebook) {
                         }
                     });
                 });
-
                 comm.send({
                     method: 'request_state'
                 }, that.callbacks());
@@ -68,38 +60,40 @@ var WidgetManager = function (comm_manager, notebook) {
                 return that.new_model({
                     model_name: widget_info.msg.content.data.state._model_name,
                     model_module: widget_info.msg.content.data.state._model_module,
+                    model_module_version: widget_info.msg.content.data.state._model_module_version,
                     comm: widget_info.comm,
                 }, widget_info.msg.content.data.state);
             }));
-        }).then(function(models) {
-
-            // Load the view information from the notebook metadata.
-            if (WidgetManager._load_callback) {
-                WidgetManager._load_callback.call(that).then(function(state) {
-                    var filtered_state = Object.keys(state).reduce(function(obj, key) {
-                        // Filter for keys that are live model ids.
-                        if (that.get_model(key)) {
-                            obj[key] = state[key];
+        }).then(function() {
+            var cells = that.notebook.get_cells();
+            var outputs, cell;
+            for (var i = 0; i < cells.length; ++i) {
+                cell = cells[i];
+                if (cell.output_area) {
+                    outputs = cell.output_area.outputs;
+                    for (var j = 0; j < outputs.length; ++j) {
+                        var out = outputs[j];
+                        if (out.output_type==="display_data" && out.data['application/vnd.jupyter.widget-view+json']) {
+                            var model_promise = that.get_model(out.data['application/vnd.jupyter.widget-view+json'].model_id);
+                            if (model_promise !== undefined) {
+                                model_promise.then((function(cell_index) {
+                                    return function (model) {
+                                        that.display_model(undefined, model, { cell_index: cell_index });
+                                    };
+                                })(i));
+                            }
                         }
-                        return obj;
-                    }, {});
-                    that.set_state(filtered_state);
-                }).catch(widgets.reject('Error loading widget manager state', true));
+                    }
+                }
             }
         });
     });
 
     // Setup state saving code.
     this.notebook.events.on('before_save.Notebook', (function() {
-
-        // Append snapshots of the widgets to the notebook's state before
-        // saving the notebook.
-        this.prependSnapshots();
-
         var save_callback = WidgetManager._save_callback;
-        var options = WidgetManager._get_state_options;
         if (save_callback) {
-            this.get_state(options).then((function(state) {
+            this.get_state(WidgetManager._get_state_options).then((function(state) {
                 save_callback.call(this, state);
             }).bind(this)).catch(widgets.reject('Could not call widget save state callback.', true));
         }
@@ -122,22 +116,12 @@ var WidgetManager = function (comm_manager, notebook) {
         validate();
     });
 
-    // Delete the snapshots when the notebook has saved, failed to save, is
-    // loaded from disk, and when the widget manager is constructed.
-    this.notebook.events.on('notebook_saved.Notebook', this.deleteSnapshots.bind(this));
-    this.notebook.events.on('notebook_save_failed.Notebook', this.deleteSnapshots.bind(this));
-    this.notebook.events.on('notebook_loaded.Notebook', this.deleteSnapshots.bind(this));
-    this.deleteSnapshots();
-
     // Create the actions and menu
     this._init_actions();
     this._init_menu();
-
-    // Initialize the widget screenshot rendering dialog.
-    this.progressModal = new progressModal.ProgressModal();
 };
-WidgetManager.prototype = Object.create(widgets.ManagerBase.prototype);
 
+WidgetManager.prototype = Object.create(widgets.ManagerBase.prototype);
 WidgetManager._managers = []; /* List of widget managers */
 WidgetManager._load_callback = null;
 WidgetManager._save_callback = null;
@@ -188,48 +172,34 @@ WidgetManager.set_state_callbacks = function (load_callback, save_callback, opti
     });
 };
 
-// Use local storage to persist widgets across page refresh by default.
-// LocalStorage is per domain, so we need to explicitly set the URL
-// that the widgets are associated with so they don't show on other
-// pages hosted by the noteboook server.
 var url = [window.location.protocol, '//', window.location.host, window.location.pathname].join('');
 var key = 'widgets:' + url;
 WidgetManager.set_state_callbacks(function() {
     if (Jupyter.notebook.metadata.widgets) {
         return Promise.resolve(Jupyter.notebook.metadata.widgets.state);
+    } else {
+        return Promise.resolve({});
     }
-    return Promise.resolve({});
-}, function(state) {
-    var stateToSave = {};
-    _.mapObject(state, function(widgetState, key) {
-        // don't persist widget state with no views
-        if (widgetState.views.length === 0) return;
-        // Only persist the views of the widget
-        stateToSave[key] = _.pick(widgetState, 'views');
-    });
-
-    // check if there are any views to persist
-    if (Object.keys(stateToSave).length === 0) {
-        // no widget state, don't save empty widget state in metadata
-        delete Jupyter.notebook.metadata.widgets;
-        return;
-    }
-
-    Jupyter.notebook.metadata.widgets = {
-        state: stateToSave,
-        // Persisted widget state version
-        version: version,
-    };
 });
 
+WidgetManager.prototype.loadClass = function(className, moduleName, moduleVersion, error) {
+    if (moduleName === "jupyter-js-widgets") {
+        if (className === "OutputModel" || className === "OutputView") {
+            return Promise.resolve(output[className]);
+        } else {
+            return Promise.resolve(widgets[className]);
+        }
+    } else {
+        return Object.getPrototypeOf(WidgetManager.prototype).loadClass.apply(this, arguments);
+    }
+}
+
 WidgetManager.prototype._handle_display_view = function (view) {
     /**
      * Have the IPython keyboard manager disable its event
      * handling so the widget can capture keyboard input.
      * Note, this is only done on the outer most widgets.
      */
-    view.trigger('displayed');
-
     if (this.keyboard_manager) {
         this.keyboard_manager.register_events(view.el);
 
@@ -245,17 +215,28 @@ WidgetManager.prototype._handle_display_view = function (view) {
  * Registers manager level actions with the notebook actions list
  */
 WidgetManager.prototype._init_actions = function() {
-    var notifier = Jupyter.notification_area.new_notification_widget('widgets');
-    this.buildSnapshotsAction = {
+    var notifier = Jupyter.notification_area.widget('widgets');
+    this.saveWidgetsAction = {
         handler: (function() {
-            this.updateSnapshots().then((function() {
-                notifier.set_message('Widgets rendered', 3000);
-            }).bind(this));
+            this.get_state({
+                drop_defaults: true
+            }).then(function(state) {
+                Jupyter.notebook.metadata.widgets = {
+                    'application/vnd.jupyter.widget-state+json' : {
+                        version_major: 1,
+                        version_minor: 0,
+                        state: state
+                    }
+                };
+                Jupyter.menubar.actions.get('jupyter-notebook:save-notebook').handler({
+                    notebook: Jupyter.notebook
+                });
+            });
         }).bind(this),
         icon: 'fa-truck',
-        help: 'Rasterizes the current state of the widgets to the notebook as PNG images.'
+        help: 'Save the notebook with the widget state information for static rendering'
     };
-    Jupyter.menubar.actions.register(this.buildSnapshotsAction, 'save-with-snapshots', 'widgets');
+    Jupyter.menubar.actions.register(this.saveWidgetsAction, 'save-with-widgets', 'widgets');
 };
 
 /**
@@ -281,9 +262,9 @@ WidgetManager.prototype._init_menu = function() {
     widgetsSubmenu.classList.add('dropdown-menu');
     widgetsMenu.appendChild(widgetsSubmenu);
 
-    widgetsSubmenu.appendChild(this._createMenuItem('Save notebook with snapshots', this.buildSnapshotsAction));
-    widgetsSubmenu.appendChild(this._createMenuItem('Download widget state', saveState.action));
-    widgetsSubmenu.appendChild(this._createMenuItem('Embed widgets', embedWidgets.action));
+    widgetsSubmenu.appendChild(this._createMenuItem('Save Notebook with Widgets', this.saveWidgetsAction));
+    widgetsSubmenu.appendChild(this._createMenuItem('Download Widget State', saveState.action));
+    widgetsSubmenu.appendChild(this._createMenuItem('Embed Widgets', embedWidgets.action));
 };
 
 /**
@@ -308,53 +289,39 @@ WidgetManager.prototype._createMenuItem = function(title, action) {
 WidgetManager.prototype.display_model = function(msg, model, options) {
     options = options || {};
     if (msg) {
-        var cell = this.get_msg_cell(msg.parent_header.msg_id);
-        if (cell === null) {
-            return Promise.reject(new Error("Could not determine where the display" +
-                " message was from.  Widget will not be displayed"));
-        } else {
-            options.cell = cell; // TODO: Handle in get state
-            // Only set cell_index when view is displayed as directly.
-            options.cell_index = this.notebook.find_cell_index(cell);
-
-            return widgets.ManagerBase.prototype.display_model.call(this, msg, model, options)
-                .catch(widgets.reject('Could not display model', true));
-        }
+        options.cell = this.get_msg_cell(msg.parent_header.msg_id);
+        // Only set cell_index when view is displayed as directly.
+        options.cell_index = this.notebook.find_cell_index(options.cell);
     } else if (options && options.cell_index !== undefined) {
         options.cell = this.notebook.get_cell(options.cell_index);
-        return widgets.ManagerBase.prototype.display_model.call(this, msg, model, options)
-            .catch(widgets.reject('Could not display model', true));
     } else {
-        return Promise.reject(new Error("Could not determine which cell the " +
-        "widget belongs to.  Widget will not be displayed"));
+        options.cell = null;
     }
+    return widgets.ManagerBase.prototype.display_model.call(this, msg, model, options)
+        .catch(widgets.reject('Could not display model', true));
 };
 
 // In display view
 WidgetManager.prototype.display_view = function(msg, view, options) {
-    if (options.cell === null) {
-        view.remove();
-        return Promise.reject(new Error("Could not determine where the display" +
-            " message was from.  Widget will not be displayed"));
-    } else {
-        if (options.cell.widgetarea) {
-            var that = this;
-            return options.cell.widgetarea.display_widget_view(Promise.resolve(view)).then(function(view) {
-                that._handle_display_view(view);
-                return view;
-            }).catch(widgets.reject('Could not display view', true));
+    if (view instanceof widgets.DOMWidgetView) {
+        if (options.cell === null) {
+            view.remove();
+            return Promise.reject(new Error("Could not determine where the display" +
+                " message was from.  Widget will not be displayed"));
         } else {
-            return Promise.reject(new Error('Cell does not have a `widgetarea` defined'));
+            if (options.cell.widgetarea) {
+                var that = this;
+                return options.cell.widgetarea.display_widget_view(Promise.resolve(view)).then(function(view) {
+                    that._handle_display_view(view);
+                    return view;
+                }).catch(widgets.reject('Could not display view', true));
+            } else {
+                return Promise.reject(new Error('Cell does not have a `widgetarea` defined'));
+            }
         }
     }
 };
 
-WidgetManager.prototype.filterViewOptions = function (options) {
-    return {
-        cell_index : options.cell_index
-    };
-};
-
 WidgetManager.prototype.setViewOptions = function (options) {
     var options = options || {};
     // If a view is passed into the method, use that view's cell as
@@ -392,235 +359,6 @@ WidgetManager.prototype.get_msg_cell = function (msg_id) {
     return null;
 };
 
-/**
- * Updates rendered snapshots of all of the widget areas in the notebook.
- * @return {Promise<void>} success
- */
-WidgetManager.prototype.updateSnapshots = function() {
-    var that = this;
-    var site = document.querySelector('#site');
-
-    // Wait for the progress modal to show before continuing
-    return this.progressModal.show().then(function() {
-        // Disable overflow to prevent the document from having elements
-        // that are scrolled out of visibility.
-        site.style.overflow = 'visible';
-        document.body.style.overflow = 'hidden';
-        // Render the widgets of each cell.
-        that._progress = 0;
-        var cells = Jupyter.notebook.get_cells();
-        return that._sequentialPromise(cells.map(that._updateCellSnapshots.bind(that)));
-    }).then(function() {
-        // When all of the rendering is complete, re-enable scrolling in the
-        // notebook.
-        site.style.overflow = '';
-        document.body.style.overflow = '';
-        // When the entire process has completed, hide the progress modal.
-        return that.progressModal.hide();
-    }).then(function() {
-        // Reset the values of the modal
-        that.progressModal.setText('Rendering widgets...');
-        return that.progressModal.setValue(0);
-    }).then(function() {
-        // Invoke a notebook save
-        Jupyter.menubar.actions.get('jupyter-notebook:save-notebook').handler({notebook: Jupyter.notebook});
-    }).catch(widgets.reject('Could not create widget snapshots and save the notebook', true));
-};
-
-/**
- * Update the widget snapshots for a single cell
- * @param  {object} cell
- * @param  {number} index - of the cell
- * @return {Promise}
- */
-WidgetManager.prototype._updateCellSnapshots = function(cell, index) {
-    var that = this;
-    var widgetSubarea = cell.element[0].querySelector(".widget-subarea");
-    var cells = Jupyter.notebook.get_cells();
-    if (!(widgetSubarea && widgetSubarea.children.length > 0)) {
-        if (widgetSubarea && widgetSubarea.widgetSnapshot) {
-            delete widgetSubarea.widgetSnapshot;
-        }
-        return that.progressModal.setValue(++that._progress/cells.length);
-    }
-
-    return that.progressModal.setText(
-        'Rendering widget ' + String(index + 1) + '/' + String(cells.length) + ' ...'
-    ).then(function() {
-        return Promise.all([
-            that._rasterizeEl(widgetSubarea),
-            that._getCellWidgetStates(cell, index),
-        ]);
-    }).then(function(results) {
-        var canvas = results[0];
-        var widgetState = results[1];
-        // Remove URL information, so only the b64 encoded data
-        // exists, because that's what the notebook likes.
-        var imageMimetype = 'image/png';
-        var imageDataUrl = canvas.toDataURL(imageMimetype);
-        var imageData = imageDataUrl.split(',').slice(-1)[0];
-
-        // Create a mime bundle.
-        var bundle = {};
-        bundle[imageMimetype] = imageData;
-        bundle['text/html'] = widgets.generateEmbedScript(widgetState, imageDataUrl);
-        widgetSubarea.widgetSnapshot = bundle;
-    }).then(function() {
-        return that.progressModal.setValue(++that._progress/cells.length);
-    });
-};
-
-WidgetManager.prototype._getCellWidgetStates = function(cell, index) {
-    var that = this;
-    var modelIds = Object.keys(this._models);
-    return Promise.all(modelIds.map(function(modelId) {
-        return that._models[modelId].then(function(model) {
-            return widgets.resolvePromisesDict(model.views).then(function(views) {
-                if (Object.keys(views).some(function(k) {
-                    return views[k].options.cell_index == index;
-                })) {
-                    return that._traverseWidgetTree(model)
-                        .map(function(widget) {
-                            return widget.get_state(true);
-                        });
-                }
-                return [null];
-            });
-        });
-    })).then(function(states) {
-        return states
-            .reduce(function(a, b) { return a.concat(b); }, [])
-            .filter(function(state) { return state !== null; });
-    });
-};
-
-WidgetManager.prototype._traverseWidgetTree = function(parentWidget, cache) {
-    // Setup a cache if it doesn't already exist
-    if (!cache) {
-        cache = {};
-    }
-
-    // Don't continue if this widget has already been traversed
-    if (parentWidget.id in cache) {
-        return [];
-    }
-
-    // Remember that this widget has been traversed
-    cache[parentWidget.id] = true;
-
-    // Traverse the parent widget's state for child widgets
-    var state = parentWidget.get_state(true);
-    var subWidgets = this._traverseWidgetState(state);
-    var that = this;
-    return _.flatten(subWidgets.map(function(subWidget) {
-        return that._traverseWidgetTree(subWidget, cache);
-    })).concat([ parentWidget ]);
-};
-
-WidgetManager.prototype._traverseWidgetState = function(state) {
-    var that = this;
-    if (state instanceof widgets.WidgetModel) {
-        return [state];
-    } else if (_.isArray(state)) {
-        return _.flatten(state.map(this._traverseWidgetState.bind(this)));
-    } else if (state instanceof Object) {
-        var states = [];
-        _.each(state, function(value, key) {
-            states.push(that._traverseWidgetState(value));
-        });
-        return _.flatten(states);
-    } else {
-        return [];
-    }
-};
-
-/**
- * Create a sequentially executed promise chain from an array of promises.
- * @param  {Array<Promise>} promises
- * @return {Promise}
- */
-WidgetManager.prototype._sequentialPromise = function(promises) {
-    var chain = Promise.resolve();
-    promises.forEach(function(promise) {
-        chain = chain.then(function() {
-            return promise;
-        });
-    });
-    return chain;
-};
-
-/**
- * Rasterize an HTMLElement to and HTML5 canvas
- * @param  {HTMLElement} el
- * @return {Promise<HTMLCanvasElement>}
- */
-WidgetManager.prototype._rasterizeEl = function(el) {
-    return new Promise(function(resolve) {
-        html2canvas(el, {
-            onrendered: function(canvas) {
-                resolve(canvas);
-            }
-        });
-    });
-};
-
-
-/**
- * Render the widget views that are live as images and prepend them as
- * outputs.
- *
- * Note: This function must be synchronous, in order to work with the
- * notebook's save machinery.
- */
-WidgetManager.prototype.prependSnapshots = function() {
-    var cells = Jupyter.notebook.get_cells();
-    cells.forEach((function(cell) {
-        var widgetSubarea = cell.element[0].querySelector(".widget-subarea");
-        if (widgetSubarea && widgetSubarea.children.length > 0 && widgetSubarea.widgetSnapshot) {
-            // Create an output for the screenshot.
-            var output = {
-                data: widgetSubarea.widgetSnapshot,
-                output_type: "display_data",
-                metadata: {isWidgetSnapshot: true}
-            };
-
-            // Move the new output to the top, so it appears
-            // where the widget area appears.
-            var outputState = cell.output_area.outputs;
-            outputState.splice(0,0,output);
-            cell.output_area.outputs = outputState;
-        }
-    }).bind(this));
-};
-
-/**
- * Remove the outputs that rendered widget view's.
- *
- * Note: This function must be synchronous, in order to work with the
- * notebook's save machinery.
- */
-WidgetManager.prototype.deleteSnapshots = function() {
-    var cells = Jupyter.notebook.get_cells();
-    cells.forEach((function(cell) {
-
-        // Remove the outputs with isWidgetSnapshot: true.
-        if (cell.output_area) {
-            // Filter out outputs that are snapshots
-            // Only touch the data, not the page
-            cell.output_area.outputs = cell.output_area.outputs.filter(function (output) {
-                return !(output.metadata && output.metadata.isWidgetSnapShot);
-            });
-
-            // Remove the corresponding elements from the page
-            cell.output_area.element
-              .find('img.jupyter-widget')
-              .each(function (index, el) {
-                el.remove();
-              });
-        }
-    }).bind(this));
-};
-
 WidgetManager.prototype._create_comm = function(comm_target_name, model_id, data) {
     var that = this;
     return this._get_connected_kernel().then(function(kernel) {
@@ -639,7 +377,6 @@ WidgetManager.prototype._create_comm = function(comm_target_name, model_id, data
     });
 };
 
-
 WidgetManager.prototype.callbacks = function (view) {
     /**
      * callback handlers specific a view
@@ -675,7 +412,6 @@ WidgetManager.prototype.callbacks = function (view) {
     return callbacks;
 };
 
-
 WidgetManager.prototype._get_comm_info = function() {
     /**
      * Gets a promise for the valid widget models.
